//  *****************************************************************************
//   						main.c
// 
//              Demonstracny program pre dosku KTK ARM7
//
//              Uspanie procesora a zobudenie stlacenim tlacidla
//
//  *****************************************************************************

// Hlavickove subory
#include "board.h"

#define PIO_INTERRUPT_LEVEL 0       // 0 - najnizsia priorita prerusenia


//  *******************************************************
//  Obsluha prerusenia od tlacidla
//  *******************************************************
void PIO_prerusenie(void)
{
  // precitanie stavu preruseni kvoli povoleniu opetovneho prerusenia
  AT91C_BASE_PIOA->PIO_ISR;
}

//  *******************************************************
//                Hlavny program
//  *******************************************************
int main(void) 
{
  //******************* Uvodna inicializacia *******************

  // Povolenie Resetu - minimalna dlzka 960 us
  AT91C_BASE_RSTC->RSTC_RMR = AT91C_RSTC_URSTEN | (0x4<<8) | (unsigned int)(0xA5<<24);
	
  //******************* Inicializacia portu A ******************

  // Povolenie hodin pre PIOA
  AT91C_BASE_PMC->PMC_PCER = (unsigned int) 1 << AT91C_ID_PIOA; 
	
  // konfiguracia jednotlivych pinov
  // vstupne piny
  AT91C_BASE_PIOA->PIO_ODR = S2;
  // vystupne piny
  AT91C_BASE_PIOA->PIO_OER = D1;
  // povolenie Pull-Up rezistoru
  AT91C_BASE_PIOA->PIO_PPUER = S2;
  // povolenie vsetkych nakonfigurovanych pinov
  AT91C_BASE_PIOA->PIO_PER = D1 | S2;
  // dioda je vypnuta
  AT91C_BASE_PIOA->PIO_CODR = D1;

  //******************* Nastavenie prerusenia od PIO ******************
  // nastavenie obsluzneho programu
  AT91C_BASE_AIC->AIC_SVR[AT91C_ID_PIOA] = (unsigned int) PIO_prerusenie;
  // zdroj prerusenia a priorita
  AT91C_BASE_AIC->AIC_SMR[AT91C_ID_PIOA] = PIO_INTERRUPT_LEVEL;
  // zrusenie prerusenia (ak nejake nahodou bolo)
  AT91C_BASE_AIC->AIC_ICCR = 0x1 << AT91C_ID_PIOA;
  // povolenie prerusenia pre prislusny pin PIOA
  AT91C_BASE_PIOA->PIO_IER = S2;
  // povolenie prerusenia od PIO
  AT91C_BASE_AIC->AIC_IECR = 0x1 << AT91C_ID_PIOA;

  //******************* Hlavna cast programu *******************
  // uspanie procesora - zakazanie hodin
  AT91C_BASE_PMC->PMC_SCDR = AT91C_PMC_PCK;
    
  // zapnutie LED - procesor bol zobudeny  
  AT91C_BASE_PIOA->PIO_SODR = D1;
  // nekonecna slucka
  while(1);
}
